/* lpq.c
 *
 * (C) Angus J. C. Duggan, 1996
 *
 * LPQ client for NT; uses registry and PRINTER environment variable for
 * printer. PRINTHOST registry key and environment variable used for LPD host.
 * LPR and LPRM also provided.
 */

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "common.h"

char *program ;

static void usage(void)
{
  error("Copyright (C) Angus J. C. Duggan, 1996. See file LICENSE for details.\nUsage:\n  %s [-Pprinter] [-Hhost] [user|job...]\n", program) ;
}

int ShowQueue(char *printer, char *hostname, SOCKET printsock, va_list args)
{
  int lpdcmd = va_arg(args, int) ;
  char *list = va_arg(args, char *) ;
  unsigned char buffer[BUFSIZ] ;
  int len, column = 0 ;

  if ( !Command("%c%s %s\n", lpdcmd, printer, (list ? list : "")) )
    return 1 ;

  /* Get reply */
  do {
    int index ;

    if ( SOCKET_ERROR == (len = recv(printsock, buffer, BUFSIZ, 0)) ) {
      warning("%s: recv from %s failed, error %d\n", program, hostname,
	      WSAGetLastError()) ;
      return 1 ;
    }

    for ( index = 0 ; index < len ; index++ ) {
      char c = buffer[index] ;

      switch ( c ) {
      case '\t':
	do {
	  fputc(' ', stdout) ;
	  column++ ;
	} while ( column % 8 != 0 ) ;
	break ;
      case '\n':
	fputc(c, stdout) ;
	column = 0 ;
	break ;
      default:
	fputc(c, stdout) ;
	column++ ;
      } 
    }
  } while ( len != 0 ) ;

  return 0 ;
}

int main(int argc, char *argv[])
{
  LPDCommand lpdcmd = LPD_SHORTQ ;
  char *printer = getenv(PRINTER) ;
  char *host = DefaultHost(printer) ;
  char *list = NULL ;

  for (program = *argv++ ; --argc ; argv++) {
    if (argv[0][0] == '-') {
      switch (argv[0][1]) {
      case 'l':	/* Long */
	lpdcmd = LPD_LONGQ ;
	break;
      case 'P':	/* printer */
	printer = &(argv[0][2]) ;
	{
	  char *at = strchr(printer, '@') ;
	  if ( at ) {
	    *at = '\0' ;
	    host = at + 1 ;
	  } else
	    host = DefaultHost(printer) ;
	}
	break;
      default:
	usage();
      }
    } else {
      int len = list ? strlen(list) + 1 : 0 ;

      list = xalloc(list, len + strlen(*argv) + 1) ;
      if ( len != 0 )	/* concat string with space separators */
	list[len - 1] = ' ' ;
      strcpy(list + len, *argv) ;
    }
  }

  return LPD(printer, host, &ShowQueue, lpdcmd, list) ;
}
