/* lprm.c
 *
 * (C) Angus J. C. Duggan, 1996
 *
 * LPRM client for NT; uses registry and PRINTER environment variable for
 * printer. PRINTHOST registry key and environment variable used for LPD host.
 * LPQ and LPR also provided.
 */

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "common.h"

char *program ;

static void usage(void)
{
  error("Copyright (C) Angus J. C. Duggan, 1996. See file LICENSE for details.\nUsage:\n  %s [-Pprinter] [-Hhost] [user|job...]\n", program) ;
}

int RemoveJob(char *printer, char *hostname, SOCKET printsock, va_list args)
{
  char *user = va_arg(args, char *) ;
  char *list = va_arg(args, char *) ;

  assert(user) ;

  if ( !Command("%c%s %s %s\n", LPD_REMOVE, printer, user, (list ? list : "")) )
    return 1 ;

  return 0 ;	/* Success! */
}

int main(int argc, char *argv[])
{
  char *printer = getenv(PRINTER) ;
  char *host = DefaultHost(printer) ;
  char *user = getenv("USERNAME") ;
  char *list = NULL ;

  if ( user == NULL )
    user = getenv("USER") ;

  if ( user == NULL )
    user = DEFAULTUSER ;

  for (program = *argv++ ; --argc ; argv++) {
    if (argv[0][0] == '-') {
      switch (argv[0][1]) {
      case 'U':	/* user */
	user = &(argv[0][2]) ;
	break;
      case 'P':	/* printer */
	printer = &(argv[0][2]) ;
	{
	  char *at = strchr(printer, '@') ;
	  if ( at ) {
	    *at = '\0' ;
	    host = at + 1 ;
	  } else
	    host = DefaultHost(printer) ;
	}
	break;
      default:
	usage();
      }
    } else {
      int len = list ? strlen(list) + 1 : 0 ;

      list = xalloc(list, len + strlen(*argv) + 1) ;
      if ( len != 0 )	/* concat string with space separators */
	list[len - 1] = ' ' ;
      strcpy(list + len, *argv) ;
    }
  }

  return LPD(printer, host, &RemoveJob, user, list) ;
}
